#!/bin/bash

if [[ $(id -u) -ne 0 ]] ; then
    echo "No root privileges"
    exit 1
fi

usage() {
    echo ""
    echo "Usage: $0 [ -i | -r | -d | -m ] [ -n <name> ] [ -p ]"
    echo ""
    echo "Options:"
    echo "  -i          Fresh installation."
    echo "  -r          Drop old schema before create a new one."
    echo "              By default the password is auto generated."
    echo "  -H <addr>   The ip address of the database."
    echo "              Default: 127.0.0.1"
    echo "  -P <port>   The database port."
    echo "              Default: 5432"
    echo "  -n <name>   Database name and user name."
    echo "              Default main schema: blxdb"
    echo "              Default datastore: blxds"
    echo "  -p          The password for the database user."
    echo "              The password is read from <STDIN>."
    echo "              If no password is set, then a password is "
    echo "              generated and written to the configuration file."
    echo "  -o <file>   Filename to write the database configuration."
    echo "              The file will be overwritten."
    echo "              Default: write to STDOUT"
    echo "  -d          Install data schema"
    echo "  -m          Install main schema"
    echo ""
    exit 1
}

INSTALL=0
REINSTALL=0
READ_PW_FROM_STDIN=0
BLOONIX_USER=
DB_HOST=127.0.0.1
DB_PORT=0
DB_NAME=""
FILENAME=stdout
MAINDB=0
DATASTORE=0
DEFAULT_CONFIG_FILE=none

while getopts "irdmpH:P:n:o:" opt; do
  case ${opt} in
    i) INSTALL=1;;
    r) REINSTALL=1;;
    p) READ_PW_FROM_STDIN=1;;
    H) DB_HOST=$OPTARG;;
    P) DB_PORT=$OPTARG;;
    o) FILENAME=$OPTARG;;
    n) DB_NAME=$OPTARG;;
    m) MAINDB=1;;
    d) DATASTORE=1;;
    \?|:|*) usage;;
  esac
done

if [[ $MAINDB -eq $DATASTORE ]] ; then
    if [[ $MAINDB -eq 1 ]] ; then
        echo "ERR: mixin -d and -m is not allowed"
    else
        echo "ERR: please set -d or -m"
    fi
    usage
fi

if [[ $MAINDB -eq 1 ]] ; then
    DEFAULT_CONFIG_FILE=/etc/bloonix/database/main.conf
    if [[ ! -z $DB_NAME ]] ; then
        BLOONIX_USER=$DB_NAME
    else
        BLOONIX_USER=blxdb
    fi
fi

if [[ $DATASTORE -eq 1 ]] ; then
    DEFAULT_CONFIG_FILE=/etc/bloonix/datastore/main.conf
    if [[ ! -z $DB_NAME ]] ; then
        BLOONIX_USER=$DB_NAME
    else
        BLOONIX_USER=blxds
    fi
fi

if [[ -z $BLOONIX_USER ]] ; then
    echo "Missing database user name"
    usage
fi

if [[ -z $FILENAME ]] ; then
    echo "Missing filename."
    usage
fi

if [[ $INSTALL -eq 0 && $REINSTALL -eq 0 ]] ; then
    usage
fi

if [[ $INSTALL -eq 1 && $REINSTALL -eq 1 ]] ; then
    echo "Mixing arguments --install and --reinstall is not allowed!"
    usage
fi

if [[ $READ_PW_FROM_STDIN -eq 1 ]] ; then
    echo -n "Enter the password for user $BLOONIX_USER: "
    read -s BLOONIX_PASSWORD
    echo ""
elif [[ ! -z $BLOONIX_INIT_SCHEMA_DB_PASSWORD ]] ; then
    BLOONIX_PASSWORD=$BLOONIX_INIT_SCHEMA_DB_PASSWORD
else
    BLOONIX_PASSWORD=$(cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w 20 | head -n 1)
fi

TMP=$(mktemp)
