create table `plugin` (
  `version` varchar(20) not null,
  `service` varchar(100) not null,
  `command` varchar(100) not null,
  `category` varchar(100) not null,
  `network_service` tinyint(1) not null default '0',
  `default_location` varchar(10) not null default 'localhost',
  `subkey` varchar(20) default '0',
  `datatype` varchar(10) not null default 'none',
  `abstract` varchar(100) not null,
  `description` varchar(500) not null,
  `info` text,
  `frequency` varchar(10) not null default 'default',
  `metrics` json not null default '{}',
  `strings` json not null default '{}',
  `options` json not null default '{}',
  `options_order` json not null default '[]',
  `options_no_mixin` json not null default '[]',
  unique(`command`)
);

create table `chart` (
  `chart_id` varchar(110) not null, -- command + 10
  `command` varchar(100) not null,
  `title` varchar(100) not null,
  `chart_type` varchar(20) not null default 'line',
  `ylabel` varchar(50) not null default 'count',
  `units` varchar(50),
  `stack` tinyint(1) not null default '0',
  `series` json not null default '[]',
  unique(`chart_id`),
  foreign key (`command`) references `plugin`(`command`) on delete cascade
) engine=innodb;

create table `message_service` (
  `id` bigint not null primary key auto_increment,
  `builtin` varchar(30) not null default 'none',
  `script` varchar(30) null,
  `name` varchar(100) not null,
  `enabled` tinyint(1) not null default '1',
  `params` json not null default '{}',
  `message_template` mediumtext,
  unique(`name`)
) engine=innodb;

create table `contact` (
  `id` bigint not null primary key auto_increment,
  `name` varchar(100) not null,
  `escalation_time` integer not null default '0',
  `timezone` varchar(40) not null default 'Europe/Berlin',
  `enabled` tinyint(1) not null default '1',
  unique(`name`)
) engine=innodb;

create table `contact_message_service` (
  `id` bigint not null primary key auto_increment,
  `contact_id` bigint not null,
  `message_service_id` bigint not null,
  `enabled` tinyint(1) not null default '1',
  `send_to` varchar(200),
  `notification_level` json not null default '[0,2,3,4,5,6]',
  `timeperiods` json not null default '[`Monday - Sunday 00:00 - 23:59`]',
  foreign key (`contact_id`) references `contact`(`id`) on delete cascade,
  foreign key (`message_service_id`) references `message_service`(`id`) on delete cascade
) engine=innodb;

create table `contactgroup` (
  `id` bigint not null primary key auto_increment,
  `name` varchar(100) not null,
  `description` varchar(100),
  unique(`name`)
) engine=innodb;

create table `contact_contactgroup` (
  `contactgroup_id` bigint not null,
  `contact_id` bigint not null,
  unique (`contactgroup_id`, `contact_id`),
  foreign key (`contactgroup_id`) references `contactgroup`(`id`) on delete cascade,
  foreign key (`contact_id`) references `contact`(`id`) on delete cascade
) engine=innodb;

create table `group` (
  `id` bigint not null primary key auto_increment,
  `name` varchar(100) not null,
  `description` varchar(100),
  unique(`name`)
) engine=innodb;

create table `host` (
  `id` bigint not null primary key auto_increment,
  `hostname` varchar(100) not null,
  `ipaddr1` varchar(45),
  `ipaddr2` varchar(45),
  `ipaddr3` varchar(45),
  `ipaddr4` varchar(45),
  `description` varchar(100),
  `allow_from` json not null default '[]',
  `comment` varchar(400),
  `sysinfo` varchar(400),
  `enabled` tinyint(1) not null default '1',
  `notification_enabled` tinyint(1) not null default '1',
  `notification_interval` integer not null default '3600',
  -- de, us, ch
  `country_code` varchar(5),
  -- the location of the server
  `location_class` varchar(100),
  -- customer, project name or anything else
  `project_class` varchar(100),
  -- lsb_release, uname -a, uptime, memory, count cpu, cpu model
  `facts` json not null default '{}',
  `variables` json not null default '{}',
  `register` tinyint(1) not null default '0',
  unique(`hostname`)
) engine=innodb;

create index `host_ipaddr1_index` on `host` (`ipaddr1`);
create index `host_ipaddr2_index` on `host` (`ipaddr2`);
create index `host_ipaddr3_index` on `host` (`ipaddr3`);
create index `host_ipaddr4_index` on `host` (`ipaddr4`);
create index `host_location_class_index` on `host` (`location_class`);
create index `host_project_class_index` on `host` (`project_class`);
create index `host_country_code_index` on `host` (`country_code`);
create index `host_register_index` on `host` (`register`);

create table `host_contactgroup` (
  `contactgroup_id` bigint not null,
  `host_id` bigint not null,
  unique (`contactgroup_id`, `host_id`),
  foreign key (`contactgroup_id`) references `contactgroup`(`id`) on delete cascade,
  foreign key (`host_id`) references `host`(`id`) on delete cascade
) engine=innodb;

create index `host_contactgroup_contactgroup_id` on `host_contactgroup` (`contactgroup_id`);
create index `host_contactgroup_host_id` on `host_contactgroup` (`host_id`);

create table `host_downtime` (
  `id` bigint not null primary key auto_increment,
  `host_id` bigint not null,
  `begin` bigint,
  `end` bigint,
  `timeslice` json not null default '[]',
  `timezone` varchar(40) not null default 'Europe/Berlin',
  `description` varchar(300) not null,
  foreign key (`host_id`) references `host`(`id`) on delete cascade
) engine=innodb;

create index `host_downtime_host_id` on `host_downtime` (`host_id`);
create index `host_downtime_index` on `host_downtime` (`host_id`, `begin`, `end`);

create table `host_group` (
  `host_id` bigint not null,
  `group_id` bigint not null,
  unique (`host_id`, `group_id`),
  foreign key (`host_id`) references `host`(`id`) on delete cascade,
  foreign key (`group_id`) references `group`(`id`) on delete cascade
) engine=innodb;

create index `host_group_host_id` on `host_group` (`host_id`);
create index `host_group_group_id` on `host_group` (`group_id`);

create table `host_secret` (
  `host_id` bigint not null,
  `password` text not null,
  foreign key (`host_id`) references `host`(`id`) on delete cascade
) engine=innodb;

create index `host_secret_host_id` on `host_secret` (`host_id`);

create table `location_group` (
  `id` bigint not null primary key auto_increment,
  `name` varchar(100) not null,
  `description` varchar(100),
  `password` text not null,
  `allow_from` json not null default '[]'
) engine=innodb;

create table `host_location_group` (
  `host_id` bigint not null,
  `location_group_id` bigint not null,
  unique(`host_id`, `location_group_id`),
  foreign key (`host_id`) references `host`(`id`) on delete cascade,
  foreign key (`location_group_id`) references `location_group`(`id`) on delete cascade
) engine=innodb;

create index `host_location_group_host_id` on `host_location_group` (`host_id`);
create index `host_location_group_location_group_id` on `host_location_group` (`location_group_id`);

create table `template` (
  `id` bigint not null primary key auto_increment,
  `name` varchar(100) not null,
  `description` varchar(1000) not null default '',
  `tags` json not null default '[]',
  `variables` json not null default '{}',
  unique(`name`)
) engine=innodb;

create table `host_template` (
  `template_id` bigint not null,
  `host_id` bigint not null,
  unique (`template_id`, `host_id`),
  foreign key (`template_id`) references `template`(`id`) on delete cascade,
  foreign key (`host_id`) references `host`(`id`) on delete cascade
) engine=innodb;

create index `host_template_template_id` on `host_template` (`template_id`);
create index `host_template_host_id` on `host_template` (`host_id`);

create table `service_parameter` (
  `id` bigint not null primary key auto_increment,
  `template_id` bigint null,
  `remote_check` tinyint(1) not null default '0',
  `service_name` varchar(100) not null,
  `host_alive_check` tinyint(1) not null default '0',
  `command` varchar(100) not null,
  `command_options` json not null default '{}',
  `satellite_type` smallint not null default '0',
  `satellites` json not null default '[]',
  `description` varchar(100) not null default '',
  `interval` integer not null default '60',
  `interval_changed_at` bigint not null default '0',
  `retry_interval` smallint not null default '60',
  `attempt_max` smallint not null default '3',
  `notification_interval` integer not null default '3600',
  `fd_enabled` tinyint(1) not null default '1',
  `fd_time_range` smallint not null default '1800',
  `fd_flap_count` smallint not null default '8',
  foreign key (`template_id`) references `template`(`id`) on delete cascade,
  foreign key (`command`) references `plugin`(`command`) on delete cascade
) engine=innodb;

create index `service_parameter_template_id` on `service_parameter` (`template_id`);
create index `service_parameter_service_name` on `service_parameter` (`service_name`);
create index `service_parameter_remote_check` on `service_parameter` (`remote_check`);

create table `service` (
  `id` bigint not null primary key auto_increment,
  `host_id` bigint not null,
  `service_parameter_id` bigint not null,

  -- changed on user action
  `comment` varchar(400) not null default '',
  -- 1 = check forced
  -- 2 = new service, check forced, ignored by timeout scheduler
  `force_check` smallint default '2',
  `enabled` tinyint(1) not null default '1',
  `acknowledged` tinyint(1) not null default '0',
  `notification_enabled` tinyint(1) not null default '1',
  `variables` json not null default '{}',

  -- changed if the value changed
  `agent_version` varchar(10) not null default '0',
  `status` tinyint(10) not null default '1',
  `status_since` bigint not null default '0',
  `status_not_ok_since` bigint not null default '0',
  `attempt_counter` smallint not null default '1',
  `last_notification` bigint not null default '0',
  `notification_status` json not null default '[]',
  `highest_notification_status` tinyint(1) not null default '0',
  `highest_fd_status` tinyint(1) not null default '0',
  `status_switches` bigint not null default '0',
  `is_flapping` smallint not null default '0',
  `has_downtime` smallint not null default '0',
  `soft_recovery` tinyint(1) not null default '0',
  `tags` varchar(100),
  `last_satellite` bigint not null default '0',

  -- changed with each check interval
  `last_check` bigint not null default '0', -- 0 = waiting for initialization
  `message` text,

  unique(`host_id`, `service_parameter_id`),
  foreign key (`host_id`) references `host`(`id`) on delete cascade,
  foreign key (`service_parameter_id`) references `service_parameter`(`id`) on delete cascade
) engine=innodb;

create index `service_service_parameter_id` on `service` (`service_parameter_id`);
create index `service_host_id` on `service` (`host_id`);
create index `service_status` on `service` (`status`);

create table `service_contactgroup` (
  `contactgroup_id` bigint not null,
  `service_id` bigint not null,
  unique (`contactgroup_id`, `service_id`),
  foreign key (`contactgroup_id`) references `contactgroup`(`id`) on delete cascade,
  foreign key (`service_id`) references `service`(`id`) on delete cascade
) engine=innodb;

create index `service_contactgroup_contactgroup_id` on `service_contactgroup` (`contactgroup_id`);
create index `service_contactgroup_service_id` on `service_contactgroup` (`service_id`);

create table `service_downtime` (
  `id` bigint not null primary key auto_increment,
  `host_id` bigint not null,
  `service_id` bigint not null,
  `begin` bigint,
  `end` bigint,
  `timeslice` json not null default '[]',
  `timezone` varchar(40) not null default 'Europe/Berlin',
  `description` varchar(300) not null,
  foreign key (`host_id`) references `host`(`id`) on delete cascade,
  foreign key (`service_id`) references `service`(`id`) on delete cascade
) engine=innodb;

create index `service_downtime_host_id` on `service_downtime` (`host_id`);
create index `service_downtime_service_id` on `service_downtime` (`service_id`);

create table `user` (
  `id` bigint not null primary key auto_increment,
  `username` varchar(50) not null,
  `name` varchar(50),
  `phone` varchar(30),
  `language` char(2) not null default 'en',
  `force_password_change` tinyint(1) not null default '1',
  `last_login` bigint not null default '0',
  `locked` tinyint(1) not null default '0',
  `role` varchar(8) not null default 'user',
  `filter_by_group` tinyint(1) not null default '1',
  `comment` varchar(500),
  `timezone` varchar(40) not null default 'Europe/Berlin',
  `last_opened_dashboard` bigint not null default '0',
  `2fa_addr` varchar(50),
  `2fa_tries` smallint not null default '0',
  `stash` json not null default '{}',
  `table_settings` json not null default '{}',
  `settings` json not null default '{}',
  `style` varchar(20) not null default 'default',
  unique(`username`)
) engine=innodb;

insert into `user` (`username`, `name`, `role`, `filter_by_group`)
values ('admin', 'Administrator', 'admin', 0);

create table `user_secret` (
  `user_id` bigint not null,
  `crypt_type` tinyint(1) default '0',
  `salt` varchar(128),
  `rounds` integer default '0',
  `password` varchar(128) not null,
  `authentication_key` varchar(128),
  foreign key (`user_id`) references `user`(`id`) on delete cascade
) engine=innodb;

insert into `user_secret` (`user_id`, `crypt_type`, `rounds`, `password`, `salt`)
values (
    1, 1, 100000,
    'QrCrlik/EEWp84HcFhWBqBOsQ4Slpp4uVWNOvEXU+sUic/e9+54ReW1UpXRBnU8ccun4c+3egoW3BvJ8uTPx5w==',
    'fBBZ8GBsY2c1nVCzTbx791yaw2JDrdWu5GdGv1sMYmLOGci6KGh7cdmtayoLewAU',
);

create table `user_dashboard` (
  `id` bigint not null primary key auto_increment,
  `user_id` bigint not null,
  `name` varchar(100) not null,
  `dashlets` json not null default '[]',
  foreign key (`user_id`) references `user`(`id`) on delete cascade
) engine=innodb;

create table `user_chart` (
  `id` bigint not null primary key auto_increment,
  `user_id` bigint not null,
  `public` tinyint(1) default '0',
  `title` varchar(50) default 'n/a',
  `subtitle` varchar(50) null,
  `yaxis_label` varchar(30) null,
  `description` varchar(100) null,
  `options` json not null default '{}',
  foreign key (`user_id`) references `user`(`id`) on delete cascade
) engine=innodb;

create index `user_chart_title_index` on `user_chart` (`title`);

create table `user_group` (
  `user_id` bigint not null,
  `group_id` bigint not null,
  unique (`user_id`, `group_id`),
  foreign key (`user_id`) references `user`(`id`) on delete cascade,
  foreign key (`group_id`) references `group`(`id`) on delete cascade
) engine=innodb;

create table `user_session` (
  `sid` varchar(255) not null,
  `user_id` bigint not null,
  `expire` bigint not null,
  `stash` mediumtext,
  `2fa_code` varchar(10),
  `2fa_success` smallint not null default '0',
  unique(`sid`),
  foreign key (`user_id`) references `user`(`id`) on delete cascade
) engine=innodb;

create table `user_trusted_browser` (
  `user_id` bigint not null,
  `2fa_sid` varchar(255) not null,
  `expire` bigint not null,
  foreign key (`user_id`) references `user`(`id`) on delete cascade
) engine=innodb;

create table `user_tracking` (
  `id` bigint not null primary key auto_increment,
  `time` bigint not null,
  `user_id` bigint not null,
  `username` varchar(50) not null,
  `action` varchar(10) not null,
  `target` varchar(200),
  `message` mediumtext,
  `old_data` mediumtext,
  `new_data` mediumtext,
  foreign key (`user_id`) references `user`(`id`) on delete cascade
) engine=innodb;

create index `user_tracking_time_index` on `user_tracking` (`time`);

create table `dependency` (
  `id` bigint not null primary key auto_increment,
  `host_id` bigint not null,
  `service_id` bigint null,
  `status` json not null default '[2,3,4,5]',
  `on_host_id` bigint not null,
  `on_service_id` bigint null,
  `on_status` json not null default '[1,5]',
  unique(`service_id`, `on_service_id`),
  foreign key (`host_id`) references `host`(`id`) on delete cascade,
  foreign key (`service_id`) references `service`(`id`) on delete cascade,
  foreign key (`on_host_id`) references `host`(`id`) on delete cascade,
  foreign key (`on_service_id`) references `service`(`id`) on delete cascade
) engine=innodb;

create index `dependency_host_id` on `dependency` (`host_id`);
create index `dependency_service_id` on `dependency` (`service_id`);

create table `satellite` (
  `id` bigint not null primary key auto_increment,
  `name` varchar(50) not null,
  `description` varchar(100),
  `hostname` varchar(50) not null,
  `port` smallint not null default '5464',
  `country` varchar(50) not null,
  `authkey` varchar(500) not null
) engine=innodb;

create table `notification` (
  `id` bigint not null primary key auto_increment,
  `time` bigint default '0',
  `host_id` bigint not null,
  `message_service_name` varchar(100) not null default 'n/a',
  `contact` varchar(100) not null,
  `send_to` varchar(200) not null,
  `message` mediumtext not null,
  `success` tinyint(1) not null default '1',
  `error` text
) engine=innodb;

create index `notification_time_host_id_index` on `notification` (`time`, `host_id`);

create table `token` (
  `tid` varchar(255) not null,
  `sid` varchar(255) not null,
  `expire` bigint not null,
  `action` varchar(100)
) engine=innodb;

create table `sys_options` (
  `key` varchar(64) not null,
  `value` varchar(512) not null,
  `data_type` varchar(6) not null,
  unique(`key`)
);

create table `db_manager` (
  `time` bigint not null
);

create table `event` (
  `time` bigint not null,
  -- host_id is used to eliminate performance issue with too many joins
  `host_id` bigint not null,
  `service_id` bigint not null,
  `last_status` tinyint(1) not null,
  `status` tinyint(1) not null,
  `attempts` varchar(10),
  `tags` varchar(100),
  `duration` bigint default '0',
  `message` mediumtext,
  `debug` json
) engine=innodb
PARTITION BY RANGE (`time`) (partition p0 values less than (0));
create index `event_host_id` on `event` (`host_id`);
create index `event_service_id` on `event` (`service_id`);
create index `event_time_service_id_index` on `event` (`time`, `service_id`);

create table `regauth` (
  `id` bigint not null primary key auto_increment,
  `name` varchar(100) not null,
  `description` varchar(100),
  `tags` json not null default '[]',
  `allowed_tags` json not null default '[]',
  `authkey` varchar(1024) not null
);
create index `regauth_authkey` on `regauth` (`authkey`);

create table `regauth_contactgroup` (
  `contactgroup_id` bigint not null,
  `regauth_id` bigint not null,
  unique (`contactgroup_id`, `regauth_id`),
  foreign key (`contactgroup_id`) references `contactgroup`(`id`) on delete cascade,
  foreign key (`regauth_id`) references `regauth`(`id`) on delete cascade
);

create table `regauth_location_group` (
  `location_group_id` bigint not null,
  `regauth_id` bigint not null,
  unique (`location_group_id`, `regauth_id`),
  foreign key (`location_group_id`) references `location_group`(`id`) on delete cascade,
  foreign key (`regauth_id`) references `regauth`(`id`) on delete cascade
);

create table `regauth_template` (
  `template_id` bigint not null,
  `regauth_id` bigint not null,
  unique (`template_id`, `regauth_id`),
  foreign key (`template_id`) references `template`(`id`) on delete cascade,
  foreign key (`regauth_id`) references `regauth`(`id`) on delete cascade
);

create table `regauth_group` (
  `group_id` bigint not null,
  `regauth_id` bigint not null,
  unique (`group_id`, `regauth_id`),
  foreign key (`group_id`) references `group`(`id`) on delete cascade,
  foreign key (`regauth_id`) references `regauth`(`id`) on delete cascade
);

create table `regauth_host` (
  `regauth_id` bigint not null,
  `host_id` bigint not null,
  `hostname` varchar(100) not null,
  `password` text not null,
  `ipaddr1` varchar(45),
  `ipaddr2` varchar(45),
  `ipaddr3` varchar(45),
  `ipaddr4` varchar(45),
  `sysinfo` varchar(400),
  `country_code` varchar(5),
  `location_class` varchar(100),
  `project_class` varchar(100),
  `allow_from` json not null default '[]',
  `tags` json not null default '[]',
  `variables` json not null default '{}',
  unique(`host_id`),
  unique(`hostname`),
  foreign key (`regauth_id`) references `regauth`(`id`) on delete cascade
);

create table `host_autodiscovery` (
  `host_id` bigint not null,
  `tag` varchar(100) not null,
  `ignore` tinyint(1) not null default '0',
  unique(`host_id`, `tag`),
  foreign key (`host_id`) references `host`(`id`) on delete cascade
);

create table `chart_view` (
  `id` bigint not null primary key auto_increment,
  `user_id` bigint not null,
  `host_id` bigint,
  `name` varchar(200),
  `data` json not null default '{}',
  foreign key (`user_id`) references `user`(`id`) on delete cascade,
  foreign key (`host_id`) references `host`(`id`) on delete cascade
);

create table `keepalived` (
  `id` bigint not null primary key auto_increment,
  `me` text,
  `time` bigint
);

insert into `sys_options` (`key`, `value`, `data_type`)
values ('model_version', '12', 'int');
